#!/usr/bin/perl -w

# Add version info
my $version = "v2.8";

# Be strict to avoid messy code
use strict;

# Use FindBin module to get script directory
use FindBin;

# Use File::Path to get some file modules
use File::Path qw(make_path);

# Fix latin characters
use Encode qw(encode decode);
my $enc = "utf-8";

# Name of our xrc gui resource file
my $xrc_gui_file = "GUI/logbook.xrc";

# Disable buffering
$|=1;

# Get script directory
my $cwd = $FindBin::RealBin."/..";
# Get script filename
my $scriptname = $FindBin::Script;
# Detect the current OS
my $OS = "$^O";

# Make a variable for users homedir
my $HOME;
# If we are on windows
if ($OS =~ /MSWin32/)
{
	# Get the environment variable for USERPROFILE
	$HOME = $ENV{"USERPROFILE"};
	# Replace all / with \
	$HOME =~ s/\//\\/g;
	
	require Win32::Console;

	# you may want to use the non-instanciated form to avoid confuzion :)
	my $codepage = Win32::Console::OutputCP();
	Win32::Console::OutputCP(1252);
}
# Else we are on UNIX
else
{
	$HOME = $ENV{"HOME"};
}

# Define a hash to store Logbooks
my $logbooks_list = "Logs";

# Define a hash to store settings information
my $settings = {
	
};

# Make a variable to contain the true $cwd
my $realcwd = $FindBin::RealBin;

# Load the settings
require "$cwd/Modules/settings.pm";
# Get the settings
$settings = settings::get_settings();

# Get the logroot location
my $logroot = $settings->{logroot};
# Remove /Modules/.. so that it will look properly if the string contains it
# (if this exists it is most likely using $cwd)
#$logroot =~ s/$realcwd/$cwd/;

# If the location contains ".app" then
if ($settings->{logroot} =~ m/(.+)\/.+\.app/)
{
	my $osxpath = $1;
	# Move outside the .app folder (osx application bundle)
	$settings->{logroot} =~ s/$realcwd/$osxpath/;
	print $settings->{logroot};
}
else
{
	# Use regex to substitute $cwd with $cwd/.. (if nothing matches it will do nothing)
	$settings->{logroot} =~ s/$realcwd/$cwd/;
}

# Define a text inside the script for use
my $logbooksheader = logbooksheader();

# Try to read the logbooks file
my $logbooksexists = main::ReadFile($settings->{"logroot"}."/logbooks.pm");

# If the logbooks.pm does not exist
if ($logbooksexists =~ /^(error reading file|)$/)
{
	# Create the folderpath to the log
	make_path($settings->{"logroot"});
	
	# Create the file
	main::WriteFile($logbooksheader, ">", $settings->{"logroot"}."/logbooks.pm");
}

# Load the logbooks
require $settings->{"logroot"}."/logbooks.pm";
# Get the logbooks
$logbooks_list = logbooks::get_logbooks();


#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------

package wxTopLevelFrame;

use Wx qw[:everything];
use Wx::XRC;
# Which events shall we include
use Wx::Event qw(EVT_BUTTON EVT_LISTBOX_DCLICK);

# Use File::Path to get some file modules
use File::Path qw(make_path);

use base qw(Wx::Frame Wx::Dialog);

sub new
{
	# Create a class
	my $class = shift;
	
	# Assign class object to $self
	my $self = $class->SUPER::new;
	
	# Initialize everything
	$self->initialize;
	
	return $self;
}

sub initialize
{
	# Get pointers
	my $self = shift;
	
	# Create mutators for widgets (enter the objectname for every object here)
	$self->create_mutator
	(
		qw
		(
			xrc_resource
			logtext
			lst_presets
			btn_close
			btn_viewlogs
			btn_settings
			btn_log
			btn_about
			logbox
			logbooks
		)
	);
	
	load_xrc_gui($self);
	
	# Setup events
	set_events($self);
	
	# Setup tooltips
	set_tooltips($self);
}

sub load_xrc_gui
{
	# Get the pointers
	my $self = shift;
	
	# Get the xrc file
	my $xrc_file = "$cwd/$xrc_gui_file";
	
	# Initialize WX
	Wx::InitAllImageHandlers();
	
	# Create xrc/xml resource
	$self->xrc_resource = Wx::XmlResource->new;
	# Initialize handlers
	$self->xrc_resource->InitAllHandlers;
	# Load the xrc file
	$self->xrc_resource->Load($xrc_file);
	
	# Tell what window/frame to load
	$self->xrc_resource->LoadFrame($self,undef,"mainwindow");
}

#
#---------------------------------------- *** ----------------------------------------
#
sub SetupUI
{
	# Get pointers
	my $self = shift;
	
	# Various functions to populate the UI with data
	load_logbooks_list($self);
	load_presets($self);
}

#
#---------------------------------------- *** ----------------------------------------
#


sub set_events
{
	# Get the pointers
	my $self = shift;
	
	# Setup the events
	# EVT_BUTTON($self, Wx::XmlResource::GetXRCID('objectname'), \&function);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('btn_log'), \&btn_log_click);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('btn_close'), \&btn_close_click);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('btn_settings'), \&btn_settings_click);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('btn_viewlogs'), \&btn_viewlogs_click);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('btn_about'), \&btn_about_click);
	EVT_LISTBOX_DCLICK($self, Wx::XmlResource::GetXRCID('lst_presets'), \&lst_presets_dclick);
	
	# Find the widgets
	# $self->objectname = $self->FindWindow('objectname');
	$self->logbox = $self->FindWindow('logbox');
	$self->btn_log = $self->FindWindow('btn_log');
	$self->logtext = $self->FindWindow('logtext');
	$self->logbooks = $self->FindWindow('logbooks');
	$self->lst_presets = $self->FindWindow('lst_presets');
	$self->btn_close = $self->FindWindow('btn_close');
	$self->btn_settings = $self->FindWindow('btn_settings');
	$self->btn_viewlogs = $self->FindWindow('btn_viewlogs');
	$self->btn_about = $self->FindWindow('btn_about');	
	
	# Set default size
	$self->SetSize(550,450);
	$self->SetMinSize($self->GetSize);
	$self->SetMaxSize($self->GetSize);
	
	# Set the window icon
	my $window_icon = Wx::Icon->new;
	$window_icon->LoadFile("$cwd/GUI/win-ico.ico", wxBITMAP_TYPE_ICO);
	$self->SetIcon($window_icon);
}

#
#---------------------------------------- *** ----------------------------------------
#

# Create mutator function from "Programming Perl"
sub create_mutator
{

	my $self = shift;

	# From "Programming Perl" 3rd Ed. p338.
	for my $attribute (@_)
	{

		no strict "refs"; # So symbolic ref to typeglob works.
		no warnings;      # Suppress "subroutine redefined" warning.

		*$attribute = sub : lvalue
		{

			my $self = shift;

			$self->{$attribute} = shift if @_;
			$self->{$attribute};

		};

	}

}


### Events

sub close_clicked
{
	# Get pointers
	my ($self, $event) = @_;
	
	# Close window
	$self->Destroy();
}

#
#---------------------------------------- *** ----------------------------------------
#

sub btn_about_click
{
	# Get pointers
	my ($self, $event) = shift;
	
	# Get current time and date
	my $timestamp = localtime();
	
	# Split the timestamp so we get
	# 0 = weekday
	# 1 = month
	# 2 = day in month
	# 3 = time
	# 4 = year
	my @date = split /\s+/, $timestamp;
	
	# Create an aboutdialog object for the about info
	my $about = Wx::AboutDialogInfo->new;
	
	# Add info to the aboutdialog
    $about->SetName( 'My Simple LogBook' );
    $about->SetVersion( "$version" );
    $about->SetDescription( 'The fast, flexible and portable logbook.' );
    $about->SetCopyright( '(c) 2009-'.$date[4].' HikariKnight <mu.antilag@gmail.com>' );
    $about->SetDevelopers( ['HikariKnight <mu.antilag@gmail.com> - Lead Developer'] );
    $about->SetArtists( ['kn|ves - Icon Artist'] );
    $about->SetWebSite('http://hkprojects.weebly.com', 'Visit the Developers Homepage');
    
    # Set the window icon
	my $window_icon = Wx::Icon->new;
	$window_icon->LoadFile("$cwd/GUI/appicon.xpm", wxBITMAP_TYPE_XPM);
	$about->SetIcon($window_icon);
    
    # Show the about dialog
    Wx::AboutBox( $about );
}

#
#---------------------------------------- *** ----------------------------------------
#

sub load_logbooks_list
{
	my $self = shift;
	
	# Split the logbooks list by newlines
	my @logbooks = split /(\n|\r|\r\n|\n\r)/, $logbooks_list;
	
	# Clear the contents first
	$self->logbooks->Clear();
	
	# For each value in the logbooks hash
	foreach my $value(@logbooks)
	{
		# If $value contains no newline
		if ($value !~ /(\n|^$)/)
		{
			# Add a logbook to the choicebox
			$self->logbooks->Append("$value LogBook");
		}
	}
	# Set the coicebox to the first logbook
	$self->logbooks->SetSelection(0);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub load_presets
{
	my $self = shift;
	
	# Read and fill up the presets field
	my $presets = main::ReadFile($settings->{logroot}."/presets.conf");
	# If presets exists
	if ($presets !~ /^(error reading file|)$/)
	{
		# Move the array pointer to a string
		$presets = "@$presets";
	}
	else
	{
		# Make presets empty
		$presets = "";
	}
	
	# Remove annoying whitespaces at the start of each line
	$presets =~ s/(\n |\r |\r\n |\n\r )/\n/g;
	
	# Clear the contents first
	$self->lst_presets->Clear();
	
	# Split the presets by newline
	my @presets_array = split /\n/, $presets;
	# For each value in the array
	foreach my $preset(@presets_array)
	{
		# Append preset to list
		$self->lst_presets->Append("$preset");
	}
}

#
#---------------------------------------- *** ----------------------------------------
#

sub lst_presets_dclick
{
	my ($self, $event) = shift;
	
	# Get the preset that was doubleclicked
	my $preset = $self->lst_presets->GetStringSelection();
	
	# Replace \n with newline
	$preset =~ s/\\n/\n/g;
	
	# Append the contents of the preset to the log
	$self->logtext->AppendText($preset);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub btn_log_click
{
	my ($self, $event) = shift;
	
	# Define a hash to so that we can convert months to month numbers
	my $month = {
		Jan => "01",
		Feb => "02",
		Mar => "03",
		Apr => "04",
		May => "05",
		Mai => "05",
		Jun => "06",
		Jul => "07",
		Aug => "08",
		Sep => "09",
		Oct => "10",
		Okt => "10",
		Nov => "11",
		Dec => "12",
		Des => "12",
	};
	
	# Get current time and date
	my $timestamp = localtime();
	
	# Split the timestamp so we get
	# 0 = weekday
	# 1 = month
	# 2 = day in month
	# 3 = time
	# 4 = year
	my @date = split /\s+/, $timestamp;
	
	# Get the currently choosen logbook from the choicebox
	my $choosen_logbook = $self->logbooks->GetString($self->logbooks->GetSelection());
	
	# Split by : so we only get the path needed
	my @current_logbook = split /\ /, $choosen_logbook;
	
	# If the day in month is lower than 10
	if ($date[2] < 10)
	{
		# Append a zero infront of the date
		$date[2] = "0$date[2]";
	}

	# Remove seconds from the time
	$date[3] =~ s/:\d{2,2}$//;
	
	# Create the folderpath to the log
	make_path($settings->{"logroot"}."/$current_logbook[0]/$date[4]/".$month->{$date[1]});
	
	# Write the log
	main::WriteFile("[$date[3]]\n".$self->logtext->GetValue()."\n\n", ">>", $settings->{"logroot"}."/$current_logbook[0]/$date[4]/".$month->{$date[1]}."/$date[4].".$month->{$date[1]}.".$date[2].txt");
	
	# Empty the textfield
	$self->logtext->SetValue("");
}

#
#---------------------------------------- *** ----------------------------------------
#

sub btn_viewlogs_click
{
	my ($self, $event) = shift;
	
	# Get the currently selected logbook
	my @logbook = split / /, $self->logbooks->GetString($self->logbooks->GetSelection());
	
	# If we are on windows
	if ($OS =~ /MSWin32/)
	{
		# Execute the logviewer against that logbook
		system("\"$cwd/logviewer.exe\" $logbook[0]");
	}
	## Else if we are on macosx
	#elsif($OS =~ /darwin/)
	#{
		## Execute the logviewer against that logbook
		#system("perl \"$cwd/Modules/logviewer\" $logbook[0]");
	#}
	else
	{
		# Execute the logviewer against that logbook and make sure to use english timeformat 
		# (to avoid a segfault in Wx if certain regional time formats are used on the system)
		system("LC_TIME=en_GB.UTF-8 perl \"$cwd/logviewer\" $logbook[0]");
	}
}

#
#---------------------------------------- *** ----------------------------------------
#

sub btn_settings_click
{
	my ($self, $event) = shift;
	
	# Execute the settings script
	if ($OS =~ /MSWin32/)
	{
		system("\"$cwd/settings.exe\"");
	}
	else
	{
		system("perl \"$cwd/settings\"");
	}
	
	# Reload stuff when closed
	load_presets($self);
	
	# Display a messagebox to notify that the function is done
	Wx::MessageBox("If you changed anything with the list of logbooks,\nthen the logbook application must restart in order\nto detect the changes.", "NOTICE!", wxOK,$self);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub btn_close_click
{
	my ($self, $event) = shift;
	
	# Close the window
	$self->Destroy();
	
	# Force quit on macosx
	if ($OS =~ /darwin/)
	{
		print "Ignore the below error, it is caused by invoking\na force quit(which was necessary to avoid perl from hanging)\n\n";
		exit;
	}
}

#
#---------------------------------------- *** ----------------------------------------
#

sub set_tooltips
{
	my ($self) = @_;
		
	# Set tooltips with info about the settings
	# $self->objectname->SetToolTip("message");
	
}

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------

package application;
use base qw(Wx::App);

sub OnInit
{
	# Get pointers
	my $self = shift;
	
	# Create mainwindow(new window)
	my $mainwindow = wxTopLevelFrame->new(undef, -1);
	
	# Set mainwindo/topwindow
	$self->SetTopWindow($mainwindow);
	
	# Show the window
	my $app = $mainwindow->Show(1);
	
	# Setup the UI with data
	wxTopLevelFrame::SetupUI($mainwindow);
	
	return $app;
}

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------


package main;

my $app = application->new;
$app->MainLoop;


#
#---------------------------------------- *** ----------------------------------------
#

# Write a file from scratch(deletes previous content)
sub WriteFile
{
	# Get the passed variables
	my ($content, $writemode, $outfile) = @_;
	
	# Open the outfile for Writing/Rewrite
	open (my $FILE, "$writemode$outfile");

	# Write the content passed to the function to the file
	print $FILE "$content";
}

#
#---------------------------------------- *** ----------------------------------------
#

# Read contents from a file and put it into a pointer
sub ReadFile 
{
	# Gets passed data from the function call
	my ($filename) = @_;

	# Makes an array to keep the inputdata
	my @inputdata;

	# Opens the passed file, if error it dies with the message "Can't open filename"
	open (my $FILE, "$filename") || return "error reading file";

	# While there is something in the file
	while(<$FILE>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@inputdata, $_);
		}
		else
		{
			# Push data into the inputdata array
			push(@inputdata, encode($enc,$_));
		}
	}

	# Close the file
	close($FILE);

	# Return the pointer to the datafile inputdata
	return(\@inputdata);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub logbooksheader
{
	my @header;
	
	while (<DATA>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@header, $_);
		}
		else
		{
			# Push data into the inputdata array
			push(@header, encode($enc,$_));
		}
	}
	# Make a string and transfer the array to it
	my $list = "@header";
	
	return $list;
}

#
#---------------------------------------- *** ----------------------------------------
#

__DATA__
package logbooks;

# Detect the current OS
my $OS = "$^O";

# Fix latin characters
use Encode qw(encode decode);
my $enc = "utf-8";

# Callback, DO NOT TOUCH!
sub get_logbooks
{
	# Make an array
	my @logbooks;
	
	# While the DATA blob contains information
	while (<DATA>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@inputdata, decode($enc,$_));
		}
		else
		{
			# Push data into the inputdata array
			push(@inputdata, encode($enc,$_));
		}
	}
	# Make a string and transfer the array to it
	my $list = "@inputdata";
	
	# Remove all whitespaces
	$list =~ s/\s+/\n/g;
	
	# Return the string list
	return $list;
}
1;
#
#---------------------------------------- *** ----------------------------------------
#

# Logbook names goes below the _DATA_ line
__DATA__
default
