#!/usr/bin/perl -w

# Be strict to avoid messy code
use strict;

# Use FindBin module to get script directory
use FindBin;

# Use File::Path to get some file modules
use File::Path qw(make_path);

# Fix latin characters
use Encode qw(encode decode);
my $enc = "utf-8";

# Name of our xrc gui resource file
my $xrc_gui_file = "../GUI/settings.xrc";

# Disable buffering
#$|=1;

# Get script directory
my $cwd = $FindBin::RealBin;
# Get script filename
my $scriptname = $FindBin::Script;
# Detect the current OS
my $OS = "$^O";

# Make a variable for users homedir
my $HOME;
# If we are on windows
if ($OS =~ /MSWin32/)
{
	# Get the environment variable for USERPROFILE
	$HOME = $ENV{"USERPROFILE"};
	# Replace all / with \
	$HOME =~ s/\\/\//g;
}
# Else we are on UNIX
else
{
	$HOME = $ENV{"HOME"};
}

# Load the settings
require "$cwd/../Modules/settings.pm";
# Get the settings
my $settings = settings::get_settings();

# If the location contains ".app" then
if ($settings->{logroot} =~ m/(.+)\/.+\.app/)
{
	my $osxpath = $1;
	# Move outside the .app folder (osx application bundle)
	$settings->{logroot} =~ s/$cwd/$osxpath/;
	#$settings->{logroot} =~ s/$cwd\/\.\./$cwd\/\.\.\/\.\.\/\.\./;
}
else
{
	# Use regex to substitute $cwd with $cwd/.. (if nothing matches it will do nothing)
	$settings->{logroot} =~ s/$cwd/$cwd\/\.\./;
}

# Define a text inside the script for use
my $logbooksheader = logbooksheader();

# Try to read the logbooks file
my $logbooksexists = main::ReadFile($settings->{"logroot"}."/logbooks.pm");

# If the logbooks.pm does not exist
if ($logbooksexists =~ /^(error reading file|)$/)
{
	# Create the folderpath to the log
	make_path($settings->{"logroot"});
	
	# Create the file
	main::WriteFile($logbooksheader, ">", $settings->{"logroot"}."/logbooks.pm");
}

# Load the logbooks
require $settings->{"logroot"}."/logbooks.pm";
# Get the logbooks
my $logbooks_list = logbooks::get_logbooks();

# Remove empty strings
$logbooks_list =~ s/^\n//g;

my $settingsheader = 'package settings;

use FindBin;
my $cwd = $FindBin::RealBin;

# Make a variable for users homedir
my $HOME;
# If we are on windows
if ("$^O" =~ /MSWin32/)
{
	# Get the environment variable for USERPROFILE
	$HOME = $ENV{"USERPROFILE"};
	
	require Win32::Console;

	# Change the codepage to 1252 (latin1 compatible)
	my $codepage = Win32::Console::OutputCP();
	Win32::Console::OutputCP(1252);
}
# Else we are on UNIX
else
{
	$HOME = $ENV{"HOME"};
}

# Define a hash to store the settings information
$settings = {
';
	
my $settingsfooter = '};

sub get_settings
{
	return $settings;
}

#
#---------------------------------------- *** ----------------------------------------
#';



#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------

package wxTopLevelFrame;

use Wx qw[:everything];
use Wx::XRC;
# Which events shall we include
use Wx::Event qw(EVT_BUTTON EVT_DIRPICKER_CHANGED EVT_CHOICE EVT_TEXT);

# Use File::Path to get some file modules
use File::Path qw(make_path);

use base qw(Wx::Frame);

sub new
{
	# Create a class
	my $class = shift;
	
	# Assign class object to $self
	my $self = $class->SUPER::new;
	
	# Initialize everything
	$self->initialize;
	
	return $self;
}

sub initialize
{
	# Get pointers
	my $self = shift;
	
	# Create mutators for widgets (enter the objectname for every object here)
	$self->create_mutator
	(
		qw
		(
			xrc_resource
			presets_text
			save
			close
			rootdir_select
			presets_text
			logbooks
			presethelp
		)
	);
	
	load_xrc_gui($self);
	
	set_events($self);
	
	set_tooltips($self);
	
}

sub load_xrc_gui
{
	# Get the pointers
	my $self = shift;
	
	# Get the xrc file
	my $xrc_file = "$cwd/$xrc_gui_file";
	
	# Initialize WX
	Wx::InitAllImageHandlers();
	
	# Create xrc/xml resource
	$self->xrc_resource = Wx::XmlResource->new;
	# Initialize handlers
	$self->xrc_resource->InitAllHandlers;
	# Load the xrc file
	$self->xrc_resource->Load($xrc_file);
	
	# Tell what window/frame to load
	$self->xrc_resource->LoadFrame($self,undef,"mainwindow");
}

#
#---------------------------------------- *** ----------------------------------------
#

sub set_events
{
	# Get the pointers
	my $self = shift;
	
	# Setup the events
	# EVT_BUTTON($self, Wx::XmlResource::GetXRCID('objectname'), \&function);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('save'), \&save_clicked);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('close'), \&close_clicked);
	EVT_DIRPICKER_CHANGED($self, Wx::XmlResource::GetXRCID('rootdir_select'), \&rootdir_select_changed);

	# Find the widgets
	# $self->objectname = $self->FindWindow('objectname');
	$self->presets_text = $self->FindWindow('presets_text');
	$self->save = $self->FindWindow('save');
	$self->close = $self->FindWindow('close');
	$self->rootdir_select = $self->FindWindow('rootdir_select');
	$self->logbooks = $self->FindWindow('logbooks');
	$self->presethelp = $self->FindWindow('presethelp');
	
	# Get the logroot location
	my $logroot = $settings->{logroot};
	# Remove /Modules/.. so that it will look properly if the string contains it
	# (if this exists it is most likely using $cwd)
	$logroot =~ s/\/Modules\/\.\.//;
		
	# Adjust the $application_path if script is run from an osx application bundle
	if ($logroot =~ m/.+(\/.+\.app)/)
	{
		# Remove move outside the .app folder
		$logroot =~ s/$1\/(Contents|\.\.)\/(Resources|\.\.)//;	
	}
	
	# Set the logroot directory in the widget
	$self->rootdir_select->SetPath($logroot);
	
	# Fill up the logbooks list
	$self->logbooks->SetValue("$logbooks_list");
	
	# Read and fill up the presets field
	my $presets = main::ReadFile($settings->{logroot}."/presets.conf");
	
	# If presets exists
	if ($presets !~ /^(error reading file|)$/)
	{
		# Move the array pointer to a string
		$presets = "@$presets";
	}
	else
	{
		# Make presets empty
		$presets = "";
	}
	
	# Remove annoying whitespaces at the start of each line
	$presets =~ s/(\n |\r |\r\n |\n\r )/\n/g;
	
	# Add presets to the presets text field
	$self->presets_text->SetValue("$presets");
	
	# Change the window size
	$self->SetSize(500,450);
	$self->SetMinSize($self->GetSize);
	
	# Set window title
	$self->SetTitle("My Simple LogBook Settings");
	
	# Add presethelp text
	$self->presethelp->SetLabel("To add a preset just type in text that you want to\npredefine, if you need more than 1 line just use \\n\ninstead of a newline.");
	
	# Set the window icon
	my $window_icon = Wx::Icon->new;
	$window_icon->LoadFile("$cwd/../GUI/win-ico.ico", wxBITMAP_TYPE_ICO);
	$self->SetIcon($window_icon);
}

#
#---------------------------------------- *** ----------------------------------------
#

# Create mutator function from "Programming Perl"
sub create_mutator
{

	my $self = shift;

	# From "Programming Perl" 3rd Ed. p338.
	for my $attribute (@_)
	{

		no strict "refs"; # So symbolic ref to typeglob works.
		no warnings;      # Suppress "subroutine redefined" warning.

		*$attribute = sub : lvalue
		{

			my $self = shift;

			$self->{$attribute} = shift if @_;
			$self->{$attribute};

		};

	}

}


### Events

sub close_clicked
{
	# Get pointers
	my ($self, $event) = @_;
	
	# Close window
	$self->Destroy();
}

#
#---------------------------------------- *** ----------------------------------------
#

sub rootdir_select_changed
{
	# Get the pointers
	my ($self) = shift;
}

#
#---------------------------------------- *** ----------------------------------------
#

sub save_clicked
{
	# Get the pointers
	my $self = shift;
	
	# Convert the path to a variable
	my $pathtowrite = $self->rootdir_select->GetPath();
	
	# Pass $cwd to a new variable
	my $application_path = $cwd;
	
	# Write the settings
	# If the rootdir is not the same as the location of the logbook
	if ($self->rootdir_select->GetPath() !~ /^($cwd|$HOME)$/)
	{
		# Convert \\ to /
		$pathtowrite =~ s/\\/\//g;
		
		# Generate the logbook application path by removing /Modules from it
		$application_path =~ s/(\/|\\)Modules//;
		
		# Adjust the $application_path if script is run from an osx application bundle
		if ($application_path =~ m/(.+)\/.+\.app/)
		{
			# Remove move outside the .app folder
			$application_path = $1;
		}
		
		# Substitute anything that contains $cwd or $application_path with the variable $cwd
		$pathtowrite =~ s/($cwd|$application_path)/\$cwd/;
		# Substitute anything that contains $HOME into the variable $HOME
		$pathtowrite =~ s/$HOME/\$HOME/;
		
		# Write the absolute directory path
		main::WriteFile("$settingsheader\n\tlogroot => \"$pathtowrite\"", ">", "$cwd/settings.pm");
	}
	# Else if rootdir is $HOME
	elsif($self->rootdir_select->GetPath() =~ /^$HOME$/)
	{
		# Write the $cwd variable
		main::WriteFile("$settingsheader\n\tlogroot => \"\$HOME\"", ">", "$cwd/settings.pm");
	}
	# Else
	else
	{
		# Write the $cwd variable
		main::WriteFile("$settingsheader\n\tlogroot => \"\$cwd\"", ">", "$cwd/settings.pm");
	}

	# Write the settings.pm footer
	main::WriteFile("\n$settingsfooter", ">>", "$cwd/settings.pm");
	
	# Get the logbooks from the widget
	my $logbooks = $self->logbooks->GetValue();
	
	# Replace all whitespace with underscore
	$logbooks =~ s/\ /_/g;
	
	# Remove default from the list
	$logbooks =~ s/^default//g;
	
	# Write the logbooks list
	main::WriteFile("$logbooksheader\n$logbooks", ">", $settings->{"logroot"}."/logbooks.pm");
	
	# Convert $pathtowrite back to a filepath
	$pathtowrite =~ s/\$cwd/$application_path/;
	
	# Create the folderpath to the log
	make_path("$pathtowrite");
	
	# Write the presets
	main::WriteFile($self->presets_text->GetValue(), ">", $pathtowrite."/presets.conf");
	
	# Close window when done
	close_clicked($self);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub set_tooltips
{
	my ($self) = @_;
		
	# Set tooltips with info about the settings
	# $self->objectname->SetToolTip("message");
	
}

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------

package application;
use base qw(Wx::App);

sub OnInit
{
	# Get pointers
	my $self = shift;
	
	# Create mainwindow(new window)
	my $mainwindow = wxTopLevelFrame->new(undef, -1);
	
	# Set mainwindo/topwindow
	$self->SetTopWindow($mainwindow);
	
	# Show the window
	$mainwindow->Show(1);
}

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------


package main;

my $app = application->new;
$app->MainLoop;


# Read contents from a file and put it into a pointer
sub ReadFile 
{
	# Gets passed data from the function call
	my ($filename) = @_;

	# Makes an array to keep the inputdata
	my @inputdata;

	# Opens the passed file, if error it dies with the message "Can't open filename"
	open (my $FILE, "$filename") || return "";

	# While there is something in the file
	while(<$FILE>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@inputdata, $_);
		}
		else
		{
			# Push data into the inputdata array
			push(@inputdata, encode($enc,$_));
		}
	}

	# Close the file
	close($FILE);

	# Return the pointer to the datafile inputdata
	return(\@inputdata);
}

#
#---------------------------------------- *** ----------------------------------------
#

# Write a file from scratch(deletes previous content)
sub WriteFile
{
	# Get the passed variables
	my ($content, $writemode, $outfile) = @_;
	
	# Open the outfile for Writing/Rewrite
	open (my $FILE, "$writemode$outfile");

	# Write the content passed to the function to the file
	print $FILE "$content";
	
	close($FILE);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub logbooksheader
{
	my @header;
	
	while (<DATA>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@header, $_);
		}
		else
		{
			# Push data into the inputdata array
			push(@header, encode($enc,$_));
		}
	}
	# Make a string and transfer the array to it
	my $list = "@header";
	
	return $list;
}

#
#---------------------------------------- *** ----------------------------------------
#

__DATA__
package logbooks;

# Detect the current OS
my $OS = "$^O";

# Fix latin characters
use Encode qw(encode decode);
my $enc = "utf-8";

# Callback, DO NOT TOUCH!
sub get_logbooks
{
	# Make an array
	my @logbooks;
	
	# While the DATA blob contains information
	while (<DATA>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@inputdata, decode($enc,$_));
		}
		else
		{
			# Push data into the inputdata array
			push(@inputdata, encode($enc,$_));
		}
	}
	# Make a string and transfer the array to it
	my $list = "@inputdata";
	
	# Remove all whitespaces
	$list =~ s/\s+/\n/g;
	
	# Return the string list
	return $list;
}
1;
#
#---------------------------------------- *** ----------------------------------------
#

# Logbook names goes below the _DATA_ line
__DATA__
default
